/******************************************************************************
  File Name: StaticAssert.cpp
  Description: Implementation for Debug Macros and Support Functions.
  Author: Lucas Goodwin (lucasg@gorge.net)
  Project: Pocket PC StaticLib
  Copyright (C) 2001 Goodwin Games -- All Rights Reserved

  See StaticAssert.h for details

******************************************************************************/
#include "StaticAssert.h"

//-----------------------------------------------------------------------------
// Assert Support Functions
//-----------------------------------------------------------------------------
#if !defined(NDEBUG)

#define ASSERT_MSG_LEN	1024

//---------------------------------
//AssertLogFile Methods
//---------------------------------
//AssertLogFile CTOR
SL_Assert::AssertLogFile::AssertLogFile( TCHAR *FileName )
{
	//Open Log file
	hFile = CreateFile( FileName, GENERIC_READ | GENERIC_WRITE,
						FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_ALWAYS,
						FILE_ATTRIBUTE_NORMAL, NULL );

	//A file was opened?
	if( INVALID_HANDLE_VALUE == hFile )
	{
		return;
	}

	//If already exists, lets prepare to append by moving to end of file
	if( ERROR_ALREADY_EXISTS == GetLastError() )
	{
		DWORD  dwPtr = SetFilePointer( hFile, 0, NULL, FILE_END );

		//Did we move properly?
		if( 0xFFFFFFFF == dwPtr )
		{
			//Oops, something messed up, close the file and set hFile to
			//invalid handle
			CloseHandle( hFile );

			//Closed the file, invalidate handle
			hFile = INVALID_HANDLE_VALUE;
		}
	}
}

//AssertLogFile DTOR
SL_Assert::AssertLogFile::~AssertLogFile()
{
	//If handle is valid then, append new data and close the file
	if( INVALID_HANDLE_VALUE != hFile )
	{
		//Append data to file by extending the EOF pointer
		SetEndOfFile( hFile );

		//Close file
		CloseHandle( hFile );
	}
}

//AssertLogFile Append
BOOL SL_Assert::AssertLogFile::Append( char *szLogMessage )
{
	BOOL Result = FALSE;

	if( INVALID_HANDLE_VALUE != hFile )
	{
		DWORD BytesWritten = 0;
		Result = WriteFile( hFile, (LPCVOID)szLogMessage,
							strlen(szLogMessage), &BytesWritten, NULL ); 
	}

	return Result;
}





//---------------------------------
//Functions
//---------------------------------
//init_assert
void SL_Assert::init_assert( HWND hWnd, TCHAR *LogFileName )
{
	g_hAssertMainWnd = hWnd;
	wsprintf( g_szFileName, TEXT("%s"), LogFileName);
}


//assert
void SL_Assert::assert( int exp, TCHAR *errmsg, TCHAR *filename,
					    TCHAR *timestamp, int linenum )
{
	if( !(exp) )
	{
		int Result;
		TCHAR szErrMsg[ASSERT_MSG_LEN];

		//Error Message for Log
		wsprintf( szErrMsg, TEXT("%s\nFile: %s\nLine: %i\nCompiled: %s\n\n"),
				  errmsg, filename, linenum, timestamp );

		//Log Assert
		log_assert( szErrMsg );

		//Error Message for Message Box
		wsprintf( szErrMsg, TEXT("%s\n\nFile: %s\nLine: %i\n\nIgnore?"),
				  errmsg, filename, linenum );
		
		//Popup Message box...
		Result = MessageBox( g_hAssertMainWnd, szErrMsg,
							 TEXT( "ASSERTION FAILURE" ), MB_YESNO
							 | MB_ICONERROR | MB_SETFOREGROUND
							 | MB_DEFBUTTON1 | MB_APPLMODAL );

		//Respond based on User input to AssertBox
		switch( Result )
		{
		//Assert ignored, continue
		case IDYES:
			break;

		//Assert not ignored, Exit
		case IDNO:
			// Terminate Application
			exit(-1);
			break;
		}
	}
}


//log_assert
void SL_Assert::log_assert( TCHAR *assertmsg )
{
	AssertLogFile Log( g_szFileName );
	char *ansi_log_break = "====================\n";
	char ansi_assert_msg[ASSERT_MSG_LEN];
	SYSTEMTIME CurSysTime;

	//Append message seperator
	Log.Append( ansi_log_break );

	//Time stamp this entry
	GetSystemTime( &CurSysTime );
	sprintf( ansi_assert_msg, "%i:%i:%i - %i/%i/%i\n",
			 CurSysTime.wHour, CurSysTime.wMinute, CurSysTime.wSecond,
			 CurSysTime.wDay, CurSysTime.wMonth, CurSysTime.wYear );
	Log.Append( ansi_assert_msg );

	//Append message seperator
	Log.Append( ansi_log_break );

	//PocketPC - All strings are unicode, lets switch to Ansi
	//			 Makes reading log file on PC easier
	int Result = WideCharToMultiByte( CP_ACP, NULL, (LPCWSTR)assertmsg,
									  -1, (LPSTR)ansi_assert_msg,
									  ASSERT_MSG_LEN, NULL, NULL );

	//Did I convert from Unicode to Ansi?
	if( !Result )
	{
		//Didn't convert properly, just output unicode string
		Log.Append( (char *)assertmsg );
	}
	else
	{
		//Converted properly, append converted Ansi string to Log file
		Log.Append( ansi_assert_msg );
	}
}


#endif //NDEBUG
