/******************************************************************************
  File Name: StaticAssert.h
  Description: Header for Debug Macros and Support Functions.
  Author: Lucas Goodwin (lucasg@gorge.net)
  Project: Pocket PC StaticLib
  Copyright (C) 2001 Goodwin Games -- All Rights Reserved

  INITASSERT - Must be called before any asserts take place with in WinMain.
			   Recommended that it be placed immediately following main window
			   creation.  If an invalid window handle is passed then assert
			   message box will not be modal.

  ASSERT	 - If expression evaluates to false...
			   Breaks to debugger if attached, displays a message window
			   indicating the expression that failed, in what file, and what
			   line.  If not ignored then program terminates with code -1.
			   The assert is appended to log file indicated by INITASSERT
			   regardless of ignore.

  ASSERTE	 - Behaves same as ASSERT except expression is replaced by the
			   passed error message.


  EX:
  INITASSERT( hWnd, "/assert.log" );
  ASSERT( 1 == 0 );
  ASSERTE( 1 == 0, "Just testing ASSERTE" );

******************************************************************************/
#if !defined(__STATICLIB_STATICASSERT__)
#define __STATICLIB_STATICASSERT__

#include "StaticIncludes.h"

//-----------------------------------------------------------------------------
//Assert Macros
//-----------------------------------------------------------------------------
// Clear Macros
#ifdef ASSERT
	#undef ASSERT
	#undef ASSERTE
	#undef INITASSERT
#endif //ASSERT

//Redefine Macros
#if !defined(NDEBUG)
	//INITASSERT
	#define INITASSERT(hwnd, logfile) SL_Assert::init_assert(hwnd, logfile)
	
	//ASSERT - Output expression as string in Assert MsgBox
	#define ASSERT(exp) \
			do { \
				if( !(exp) ) \
				{ \
					DebugBreak(); \
					SL_Assert::assert(exp, TEXT(#exp), TEXT(__FILE__), \
									  TEXT(__TIMESTAMP__), __LINE__); \
				} \
			} while (0)

	//ASSERTE - Output error message in Assert MsgBox
	#define ASSERTE(exp, errmsg) \
			do { \
				if( !(exp) ) \
				{ \
					DebugBreak(); \
					SL_Assert::assert(exp, TEXT(errmsg), TEXT(__FILE__), \
									  TEXT(__TIMESTAMP__), __LINE__); \
				} \
			} while (0)
#else
	#define INITASSERT(hwnd, logfile) void(0)
	#define ASSERT(exp) void(0)
	#define ASSERTE(exp, errmsg) void(0)
#endif // DEBUG



//-----------------------------------------------------------------------------
// Assert Support Functions/Objects
//-----------------------------------------------------------------------------
#if !defined(NDEBUG)

namespace SL_Assert {
	//---------------------------------
	//Namespace Globals
	//---------------------------------
	static HWND		g_hAssertMainWnd = NULL;	//Handle to main window of app
	static TCHAR	g_szFileName[512];			//String holding file name

	//---------------------------------
	//Namespace Objects
	//---------------------------------
	class AssertLogFile
	{
		HANDLE hFile;
	public:
		AssertLogFile( TCHAR *FileName );
		~AssertLogFile();
		BOOL Append( char *szLogMessage );
	};

	//---------------------------------
	//Namespace Functions
	//---------------------------------
	//init_assert - Set Main Window and Log File Name for use in assert
	void init_assert( HWND hWnd, TCHAR *LogFileName );

	//assert - Display error message box, log assert, and exit (if so desired)
	void assert( int exp, TCHAR *errmsg, TCHAR *filename, TCHAR *timestamp,
				 int linenum );

	//log_assert - Log assert message to log file, close File-Map object
	void log_assert( TCHAR *assertmsg );
}

#endif //NDEBUG


#endif //__STATICLIB_STATICASSERT__